@@ -85,7 +85,7 @@ class FollowLog:
         # noinspection PyTypeChecker
         with open(self.workdir / 'rendering.log', 'wb') as f:
             for s in stdout:
-                # remove ASCII NULL symbol inplace
+                # remove ASCII NULL symbols inplace
                 part = s.replace(b'\x00', b'')
                 f.write(part)
                 f.flush()
@@ -97,15 +97,12 @@ class Key:
     def __init__(self, key: str):
         self.path = PurePosixPath(key)
 
-    @property
-    def task_id(self):
-        return self.path.parent
 
-    @property
-    def workdir(self):
-        directory = RENDER_DIR / self.task_id.name
-        directory.mkdir(exist_ok=True)
-        return directory
+class FileKey(Key):
+
+    def __init__(self, workdir: Path, *args, **kwargs):
+        self.workdir = workdir
+        super().__init__(*args, **kwargs)
 
     @property
     def filename(self):
@@ -116,17 +113,38 @@ class Key:
         return self.workdir / 'scene' / self.filename
 
 
-class Texture(Key):
+class TextureFileKey(FileKey):
     @property
     def dst(self):
         return self.workdir / 'maps'
 
 
+class DestKey(Key):
+    @property
+    def task_id(self):
+        return self.path
+
+    @property
+    def scene_id(self):
+        return self.task_id.parent
+
+    @property
+    def workdir(self):
+        directory = RENDER_DIR / self.scene_id.name / self.task_id.name
+        directory.mkdir(exist_ok=True)
+        return directory
+
+
 class Render:
-    def __init__(self, scene: Key, client: S3, texture: str = None):
+    DEFAULT_EXTENSION = 'png'
+    EXTENSIONS = [DEFAULT_EXTENSION, 'exr', 'cxr']
+
+    def __init__(self, scene: FileKey, dest: DestKey, client: S3,
+                 texture: TextureFileKey = None):
         self.scene = scene
+        self.dest = dest
         self.client = client
-        self.texture = texture and Texture(texture)
+        self.texture = texture
 
     def unpack_texture(self):
         if self.texture:
@@ -137,7 +155,10 @@ class Render:
                 f.extractall(self.texture.dst)
             logger.info(f"Texture unzipped to - {self.texture.dst}")
 
-    def run(self, extension='exr'):
+    def run(self, extension):
+        assert extension in self.EXTENSIONS, f"{extension} is not supported"
+        logger.debug(f"Using {extension} for output files")
+
         self.unpack_texture()
         self.client.download_file(self.scene.path, self.scene.localfile)
         logger.info(f"Scene downloaded - {self.scene.localfile}")
@@ -149,7 +170,7 @@ class Render:
             str(max_cmd),
             "-continueOnError",
             "-batchRender",
-            f'-outputName:{str(self.scene.workdir / f"0.{extension}")}',
+            f'-outputName:{str(self.dest.workdir / f"0.{extension}")}',
         ]
         if self.texture:
             logger.debug(f"Using textures: {list(self.texture.dst.glob('*'))}")
@@ -158,11 +179,11 @@ class Render:
         logger.debug(f"Params = {args}")
 
         process = Popen(args, stderr=STDOUT, stdout=PIPE, bufsize=1)
-        follow = FollowLog(self.scene.workdir, extension)
+        follow = FollowLog(self.dest.workdir, extension)
         for file in follow.iterate_files(process.stdout):
             view = file.parent.name
             self.client.upload_file(
-                self.scene.task_id / 'result' / view / file.name,
+                self.dest.task_id / 'result' / view / file.name,
                 file
             )
             logger.info(f"Artifact is uploaded - {file}")
@@ -184,38 +205,43 @@ def record_time(f):
     return wrapper
 
 
-def collect_logs(scene: Key, client: S3):
+def collect_logs(dest: DestKey, client: S3):
     logger.info("Collecting log files...")
     for file in itertools.chain(
             # additional logs could be appended here
             glob.glob(str(LOG_DIR / '*.log')),
-            glob.glob(str(scene.workdir / '*.log')),
+            glob.glob(str(dest.workdir / '*.log')),
     ):
         logger.info(f"Found log {file}")
         client.upload_file(
-            scene.task_id / 'logs' / os.path.basename(file),
+            dest.task_id / 'logs' / os.path.basename(file),
             file
         )
 
 
-def run(scene_key, texture=None):
+def run(
+        scene_key: str,
+        dest_key: str,
+        texture: str = None,
+        extension: str = Render.DEFAULT_EXTENSION
+):
+    dest = DestKey(dest_key)
+    configure_logger(dest.workdir / 'task.log')
+
     assert os.environ['ADSK_3DSMAX_x64_2018'], "ADSK_3DSMAX_x64_2018 must be defined"
 
     s3 = AWS_S3()
-    scene = Key(scene_key)
-    # configure current logger for the specific scene
-    configure_logger(scene.workdir / 'task.log')
-
-    render = Render(scene, s3, texture)
+    scene = FileKey(dest.workdir, scene_key)
+    render = Render(scene, dest, s3, texture and TextureFileKey(dest.workdir, texture))
     try:
-        rc = record_time(render.run)()
+        rc = record_time(render.run)(extension)
         if rc > 0:
             raise Exception(f"3dsmax returned non zero code {rc}")
     except Exception as e:
         logger.exception(e)
         raise
     finally:
-        collect_logs(scene, s3)
+        collect_logs(dest, s3)
         collect_instance_logs(s3)
 
 
